"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const graphql_1 = require("graphql");
const node_fetch_1 = __importDefault(require("node-fetch"));
const disparity_1 = __importDefault(require("disparity"));
const merge_graphql_schemas_1 = require("merge-graphql-schemas");
const is_glob_1 = __importDefault(require("is-glob"));
async function fetchRemoteSchema(endpoint) {
    return node_fetch_1.default(endpoint, {
        method: 'POST',
        headers: {
            'content-type': 'application/json'
        },
        body: JSON.stringify({ query: graphql_1.introspectionQuery })
    })
        .then(res => {
        if (res.ok) {
            return res;
        }
        throw new Error(res.statusText);
    })
        .then(res => res.json())
        .then(({ data }) => graphql_1.buildClientSchema(data));
}
async function readLocalSchema(schemaPath) {
    if (is_glob_1.default(schemaPath)) {
        const typesArray = merge_graphql_schemas_1.fileLoader(schemaPath);
        const mergedSchema = merge_graphql_schemas_1.mergeTypes(typesArray, { all: true });
        return Promise.resolve(graphql_1.buildSchema(mergedSchema));
    }
    else {
        const schemaString = fs_1.default.readFileSync(schemaPath, 'utf8');
        return Promise.resolve(graphql_1.buildSchema(schemaString));
    }
}
async function getSchema(schemaLocation) {
    if (schemaLocation.match(/^https?/)) {
        return fetchRemoteSchema(schemaLocation);
    }
    else {
        return readLocalSchema(schemaLocation);
    }
}
async function getDiff(schema1Location, schema2Location) {
    const [schema1, schema2] = await Promise.all([schema1Location, schema2Location].map(schemaLocation => getSchema(schemaLocation)));
    if (!schema1 || !schema2) {
        throw new Error('Schemas not defined');
    }
    const [schema1SDL, schema2SDL] = [schema1, schema2].map(schema => graphql_1.printSchema(schema) || '');
    if (schema1SDL === schema2SDL) {
        return;
    }
    const diff = disparity_1.default.unified(schema1SDL, schema2SDL, {
        paths: [schema1Location, schema2Location]
    });
    const dangerousChanges = graphql_1.findDangerousChanges(schema1, schema2);
    const breakingChanges = graphql_1.findBreakingChanges(schema1, schema2);
    return {
        diff,
        dangerousChanges,
        breakingChanges
    };
}
exports.getDiff = getDiff;
